@@ -6,13 +6,23 @@ import sys
 import time
 import builtins
 
-from xonsh.tools import (XonshError, print_exception, DefaultNotGiven,
-                         check_for_partial_string, format_std_prepost,
-                         LINE_CONTINUATION)
+from xonsh.tools import (
+    XonshError,
+    print_exception,
+    DefaultNotGiven,
+    check_for_partial_string,
+    format_std_prepost,
+    get_line_continuation,
+)
 from xonsh.platform import HAS_PYGMENTS, ON_WINDOWS
-from xonsh.codecache import (should_use_cache, code_cache_name,
-                             code_cache_check, get_cache_filename,
-                             update_cache, run_compiled_code)
+from xonsh.codecache import (
+    should_use_cache,
+    code_cache_name,
+    code_cache_check,
+    get_cache_filename,
+    update_cache,
+    run_compiled_code,
+)
 from xonsh.completer import Completer
 from xonsh.prompt.base import multiline_prompt, PromptFormatter
 from xonsh.events import events
@@ -22,6 +32,7 @@ from xonsh.ansi_colors import ansi_partial_color_format
 
 if ON_WINDOWS:
     import ctypes
+
     kernel32 = ctypes.windll.kernel32
     kernel32.SetConsoleTitleW.argtypes = [ctypes.c_wchar_p]
 
@@ -31,8 +42,9 @@ class _TeeStdBuf(io.RawIOBase):
     in memory buffer.
     """
 
-    def __init__(self, stdbuf, membuf, encoding=None, errors=None, prestd=b'',
-                 poststd=b''):
+    def __init__(
+        self, stdbuf, membuf, encoding=None, errors=None, prestd=b"", poststd=b""
+    ):
         """
         Parameters
         ----------
@@ -42,10 +54,11 @@ class _TeeStdBuf(io.RawIOBase):
             The in memory stream buffer.
         encoding : str or None, optional
             The encoding of the stream. Only used if stdbuf is a text stream,
-            rather than a binary one.
+            rather than a binary one. Defaults to $XONSH_ENCODING if None.
         errors : str or None, optional
             The error form for the encoding of the stream. Only used if stdbuf
-            is a text stream, rather than a binary one.
+            is a text stream, rather than a binary one. Deafults to
+            $XONSH_ENCODING_ERRORS if None.
         prestd : bytes, optional
             The prefix to prepend to the standard buffer.
         poststd : bytes, optional
@@ -53,11 +66,12 @@ class _TeeStdBuf(io.RawIOBase):
         """
         self.stdbuf = stdbuf
         self.membuf = membuf
-        self.encoding = encoding
-        self.errors = errors
+        env = builtins.__xonsh_env__
+        self.encoding = env.get("XONSH_ENCODING") if encoding is None else encoding
+        self.errors = env.get("XONSH_ENCODING_ERRORS") if errors is None else errors
         self.prestd = prestd
         self.poststd = poststd
-        self._std_is_binary = not hasattr(stdbuf, 'encoding')
+        self._std_is_binary = not hasattr(stdbuf, "encoding")
 
     def fileno(self):
         """Returns the file descriptor of the std buffer."""
@@ -90,15 +104,14 @@ class _TeeStdBuf(io.RawIOBase):
         if self._std_is_binary:
             self.stdbuf.write(std_b)
         else:
-            self.stdbuf.write(std_b.decode(encoding=self.encoding,
-                                           errors=self.errors))
+            self.stdbuf.write(std_b.decode(encoding=self.encoding, errors=self.errors))
         return self.membuf.write(b)
 
 
 class _TeeStd(io.TextIOBase):
     """Tees a std stream into an in-memory container and the original stream."""
 
-    def __init__(self, name, mem, prestd='', poststd=''):
+    def __init__(self, name, mem, prestd="", poststd=""):
         """
         Parameters
         ----------
@@ -118,14 +131,19 @@ class _TeeStd(io.TextIOBase):
         self.poststd = poststd
         preb = prestd.encode(encoding=mem.encoding, errors=mem.errors)
         postb = poststd.encode(encoding=mem.encoding, errors=mem.errors)
-        if hasattr(std, 'buffer'):
-            buffer = _TeeStdBuf(std.buffer, mem.buffer,
-                                prestd=preb, poststd=postb)
+        if hasattr(std, "buffer"):
+            buffer = _TeeStdBuf(std.buffer, mem.buffer, prestd=preb, poststd=postb)
         else:
             # TextIO does not have buffer as part of the API, so std streams
             # may not either.
-            buffer = _TeeStdBuf(std, mem.buffer, encoding=mem.encoding,
-                                errors=mem.errors, prestd=preb, poststd=postb)
+            buffer = _TeeStdBuf(
+                std,
+                mem.buffer,
+                encoding=mem.encoding,
+                errors=mem.errors,
+                prestd=preb,
+                poststd=postb,
+            )
         self.buffer = buffer
         setattr(sys, name, self)
 
@@ -213,28 +231,41 @@ class _TeeStd(io.TextIOBase):
 
 
 class Tee:
-    """Class that merges tee'd stdout and stderr into a single strea,.
+    """Class that merges tee'd stdout and stderr into a single stream.
 
     This represents what a user would actually see on the command line.
-    This class as the same interface as io.TextIOWrapper, except that
+    This class has the same interface as io.TextIOWrapper, except that
     the buffer is optional.
     """
+
     # pylint is a stupid about counting public methods when using inheritance.
     # pylint: disable=too-few-public-methods
 
-    def __init__(self, buffer=None, encoding=None, errors=None,
-                 newline=None, line_buffering=False, write_through=False):
+    def __init__(
+        self,
+        buffer=None,
+        encoding=None,
+        errors=None,
+        newline=None,
+        line_buffering=False,
+        write_through=False,
+    ):
         self.buffer = io.BytesIO() if buffer is None else buffer
-        self.memory = io.TextIOWrapper(self.buffer, encoding=encoding,
-                                       errors=errors, newline=newline,
-                                       line_buffering=line_buffering,
-                                       write_through=write_through)
-        self.stdout = _TeeStd('stdout', self.memory)
+        self.memory = io.TextIOWrapper(
+            self.buffer,
+            encoding=encoding,
+            errors=errors,
+            newline=newline,
+            line_buffering=line_buffering,
+            write_through=write_through,
+        )
+        self.stdout = _TeeStd("stdout", self.memory)
         env = builtins.__xonsh_env__
-        prestderr = format_std_prepost(env.get('XONSH_STDERR_PREFIX'))
-        poststderr = format_std_prepost(env.get('XONSH_STDERR_POSTFIX'))
-        self.stderr = _TeeStd('stderr', self.memory, prestd=prestderr,
-                              poststd=poststderr)
+        prestderr = format_std_prepost(env.get("XONSH_STDERR_PREFIX"))
+        poststderr = format_std_prepost(env.get("XONSH_STDERR_POSTFIX"))
+        self.stderr = _TeeStd(
+            "stderr", self.memory, prestd=prestderr, poststd=poststderr
+        )
 
     @property
     def line_buffering(self):
@@ -267,21 +298,22 @@ class BaseShell(object):
         super().__init__()
         self.execer = execer
         self.ctx = ctx
-        self.completer = Completer() if kwargs.get('completer', True) else None
+        self.completer = Completer() if kwargs.get("completer", True) else None
         self.buffer = []
         self.need_more_lines = False
         self.mlprompt = None
         self._styler = DefaultNotGiven
         self.prompt_formatter = PromptFormatter()
-        self.accumulated_inputs = ''
+        self.accumulated_inputs = ""
 
     @property
     def styler(self):
         if self._styler is DefaultNotGiven:
             if HAS_PYGMENTS:
                 from xonsh.pyghooks import XonshStyle
+
                 env = builtins.__xonsh_env__
-                self._styler = XonshStyle(env.get('XONSH_COLOR_STYLE'))
+                self._styler = XonshStyle(env.get("XONSH_COLOR_STYLE"))
             else:
                 self._styler = None
         return self._styler
@@ -297,11 +329,11 @@ class BaseShell(object):
     def emptyline(self):
         """Called when an empty line has been entered."""
         self.need_more_lines = False
-        self.default('')
+        self.default("")
 
     def singleline(self, **kwargs):
         """Reads a single line of input from the shell."""
-        msg = '{0} has not implemented singleline().'
+        msg = "{0} has not implemented singleline()."
         raise RuntimeError(msg.format(self.__class__.__name__))
 
     def precmd(self, line):
@@ -310,7 +342,7 @@ class BaseShell(object):
 
     def default(self, line):
         """Implements code execution."""
-        line = line if line.endswith('\n') else line + '\n'
+        line = line if line.endswith("\n") else line + "\n"
         src, code = self.push(line)
         if code is None:
             return
@@ -320,12 +352,12 @@ class BaseShell(object):
         env = builtins.__xonsh_env__
         hist = builtins.__xonsh_history__  # pylint: disable=no-member
         ts1 = None
-        enc = env.get('XONSH_ENCODING')
-        err = env.get('XONSH_ENCODING_ERRORS')
+        enc = env.get("XONSH_ENCODING")
+        err = env.get("XONSH_ENCODING_ERRORS")
         tee = Tee(encoding=enc, errors=err)
         try:
             ts0 = time.time()
-            run_compiled_code(code, self.ctx, None, 'single')
+            run_compiled_code(code, self.ctx, None, "single")
             ts1 = time.time()
             if hist is not None and hist.last_cmd_rtn is None:
                 hist.last_cmd_rtn = 0  # returncode for success
@@ -339,8 +371,15 @@ class BaseShell(object):
                 hist.last_cmd_rtn = 1  # return code for failure
         finally:
             ts1 = ts1 or time.time()
-            self._append_history(inp=src, ts=[ts0, ts1], tee_out=tee.getvalue())
+            tee_out = tee.getvalue()
+            self._append_history(inp=src, ts=[ts0, ts1], tee_out=tee_out)
             self.accumulated_inputs += src
+            if (
+                tee_out
+                and env.get("XONSH_APPEND_NEWLINE")
+                and not tee_out.endswith(os.linesep)
+            ):
+                print(os.linesep, end="")
             tee.close()
             self._fix_cwd()
         if builtins.__xonsh_exit__:  # pylint: disable=no-member
@@ -353,23 +392,20 @@ class BaseShell(object):
         information is available.
         """
         hist = builtins.__xonsh_history__  # pylint: disable=no-member
-        info['rtn'] = hist.last_cmd_rtn if hist is not None else None
+        info["rtn"] = hist.last_cmd_rtn if hist is not None else None
         tee_out = tee_out or None
         last_out = hist.last_cmd_out if hist is not None else None
         if last_out is None and tee_out is None:
             pass
         elif last_out is None and tee_out is not None:
-            info['out'] = tee_out
+            info["out"] = tee_out
         elif last_out is not None and tee_out is None:
-            info['out'] = last_out
+            info["out"] = last_out
         else:
-            info['out'] = tee_out + '\n' + last_out
+            info["out"] = tee_out + "\n" + last_out
         events.on_postcommand.fire(
-            cmd=info['inp'],
-            rtn=info['rtn'],
-            out=info.get('out', None),
-            ts=info['ts']
-            )
+            cmd=info["inp"], rtn=info["rtn"], out=info.get("out", None), ts=info["ts"]
+        )
         if hist is not None:
             hist.append(info)
             hist.last_cmd_rtn = hist.last_cmd_out = None
@@ -383,27 +419,27 @@ class BaseShell(object):
             cwd = None
         if cwd is None:
             # directory has been deleted out from under us, most likely
-            pwd = env.get('PWD', None)
+            pwd = env.get("PWD", None)
             if pwd is None:
                 # we have no idea where we are
-                env['PWD'] = '<invalid directory>'
+                env["PWD"] = "<invalid directory>"
             elif os.path.isdir(pwd):
                 # unclear why os.getcwd() failed. do nothing.
                 pass
             else:
                 # OK PWD is really gone.
-                msg = '{UNDERLINE_INTENSE_WHITE}{BACKGROUND_INTENSE_BLACK}'
+                msg = "{UNDERLINE_INTENSE_WHITE}{BACKGROUND_INTENSE_BLACK}"
                 msg += "xonsh: working directory does not exist: " + pwd
-                msg += '{NO_COLOR}'
+                msg += "{NO_COLOR}"
                 self.print_color(msg, file=sys.stderr)
-        elif 'PWD' not in env:
+        elif "PWD" not in env:
             # $PWD is missing from env, recreate it
-            env['PWD'] = cwd
-        elif os.path.realpath(cwd) != os.path.realpath(env['PWD']):
+            env["PWD"] = cwd
+        elif os.path.realpath(cwd) != os.path.realpath(env["PWD"]):
             # The working directory has changed without updating $PWD, fix this
-            old = env['PWD']
-            env['PWD'] = cwd
-            env['OLDPWD'] = old
+            old = env["PWD"]
+            env["PWD"] = cwd
+            env["OLDPWD"] = old
             events.on_chdir.fire(olddir=old, newdir=cwd)
 
     def push(self, line):
@@ -413,7 +449,7 @@ class BaseShell(object):
         self.buffer.append(line)
         if self.need_more_lines:
             return None, None
-        src = ''.join(self.buffer)
+        src = "".join(self.buffer)
         src = transform_command(src)
         return self.compile(src)
 
@@ -421,7 +457,7 @@ class BaseShell(object):
         """Compiles source code and returns the (possibly modified) source and
         a valid code object.
         """
-        _cache = should_use_cache(self.execer, 'single')
+        _cache = should_use_cache(self.execer, "single")
         if _cache:
             codefname = code_cache_name(src)
             cachefname = get_cache_filename(codefname, code=True)
@@ -429,22 +465,21 @@ class BaseShell(object):
             if usecache:
                 self.reset_buffer()
                 return src, code
-        if src.endswith(str(LINE_CONTINUATION)+'\n'):
+        lincont = get_line_continuation()
+        if src.endswith(lincont + "\n"):
             self.need_more_lines = True
             return src, None
         try:
-            code = self.execer.compile(src,
-                                       mode='single',
-                                       glbs=self.ctx,
-                                       locs=None)
+            code = self.execer.compile(src, mode="single", glbs=self.ctx, locs=None)
             if _cache:
                 update_cache(code, cachefname)
             self.reset_buffer()
         except SyntaxError:
             partial_string_info = check_for_partial_string(src)
-            in_partial_string = (partial_string_info[0] is not None and
-                                 partial_string_info[1] is None)
-            if (src == '\n' or src.endswith('\n\n')) and not in_partial_string:
+            in_partial_string = (
+                partial_string_info[0] is not None and partial_string_info[1] is None
+            )
+            if (src == "\n" or src.endswith("\n\n")) and not in_partial_string:
                 self.reset_buffer()
                 print_exception()
                 return src, None
@@ -465,21 +500,24 @@ class BaseShell(object):
     def settitle(self):
         """Sets terminal title."""
         env = builtins.__xonsh_env__  # pylint: disable=no-member
-        term = env.get('TERM', None)
+        term = env.get("TERM", None)
         # Shells running in emacs sets TERM to "dumb" or "eterm-color".
         # Do not set title for these to avoid garbled prompt.
-        if (term is None and not ON_WINDOWS) or term in ['dumb', 'eterm-color',
-                                                         'linux']:
+        if (term is None and not ON_WINDOWS) or term in [
+            "dumb",
+            "eterm-color",
+            "linux",
+        ]:
             return
-        t = env.get('TITLE')
+        t = env.get("TITLE")
         if t is None:
             return
         t = self.prompt_formatter(t)
-        if ON_WINDOWS and 'ANSICON' not in env:
+        if ON_WINDOWS and "ANSICON" not in env:
             kernel32.SetConsoleTitleW(t)
         else:
-            with open(1, 'wb', closefd=False) as f:
-                # prevent xonsh from answering interative questions
+            with open(1, "wb", closefd=False) as f:
+                # prevent xonsh from answering interactive questions
                 # on the next command by writing the title
                 f.write("\x1b]0;{0}\x07".format(t).encode())
                 f.flush()
@@ -493,10 +531,10 @@ class BaseShell(object):
                     self.mlprompt = multiline_prompt()
                 except Exception:  # pylint: disable=broad-except
                     print_exception()
-                    self.mlprompt = '<multiline prompt error> '
+                    self.mlprompt = "<multiline prompt error> "
             return self.mlprompt
         env = builtins.__xonsh_env__  # pylint: disable=no-member
-        p = env.get('PROMPT')
+        p = env.get("PROMPT")
         try:
             p = self.prompt_formatter(p)
         except Exception:  # pylint: disable=broad-except
@@ -505,15 +543,15 @@ class BaseShell(object):
         return p
 
     def format_color(self, string, hide=False, force_string=False, **kwargs):
-        """Formats the colors in a string. This base implmentation colors based
-        on ANSI color codes
+        """Formats the colors in a string. ``BaseShell``'s default implementation
+        of this method uses colors based on ANSI color codes.
         """
-        style = builtins.__xonsh_env__.get('XONSH_COLOR_STYLE')
+        style = builtins.__xonsh_env__.get("XONSH_COLOR_STYLE")
         return ansi_partial_color_format(string, hide=hide, style=style)
 
     def print_color(self, string, hide=False, **kwargs):
-        """Prints a string in color. This base implmentation will color based
-        ANSI color codes if a string was given as input. If a list of token
+        """Prints a string in color. This base implementation's colors are based
+        on ANSI color codes if a string was given as input. If a list of token
         pairs is given, it will color based on pygments, if available. If
         pygments is not available, it will print a colorless string.
         """
@@ -522,13 +560,13 @@ class BaseShell(object):
         elif HAS_PYGMENTS:
             # assume this is a list of (Token, str) tuples and format it
             env = builtins.__xonsh_env__
-            self.styler.style_name = env.get('XONSH_COLOR_STYLE')
+            self.styler.style_name = env.get("XONSH_COLOR_STYLE")
             style_proxy = pyghooks.xonsh_style_proxy(self.styler)
             formatter = pyghooks.XonshTerminal256Formatter(style=style_proxy)
             s = pygments.format(string, formatter).rstrip()
         else:
             # assume this is a list of (Token, str) tuples and remove color
-            s = ''.join([x for _, x in string])
+            s = "".join([x for _, x in string])
         print(s, **kwargs)
 
     def color_style_names(self):
