@@ -1,19 +1,23 @@
 import builtins
+import collections
 
-from collections import OrderedDict
+import xonsh.lazyasd as xl
 
 from xonsh.completers.tools import justify
 
-VALID_ACTIONS = frozenset({'add', 'remove', 'list'})
+
+VALID_ACTIONS = xl.LazyObject(
+    lambda: frozenset({"add", "remove", "list"}), globals(), "VALID_ACTIONS"
+)
 
 
-def _add_one_completer(name, func, loc='end'):
-    new = OrderedDict()
-    if loc == 'start':
+def _add_one_completer(name, func, loc="end"):
+    new = collections.OrderedDict()
+    if loc == "start":
         new[name] = func
         for (k, v) in builtins.__xonsh_completers__.items():
             new[k] = v
-    elif loc == 'end':
+    elif loc == "end":
         for (k, v) in builtins.__xonsh_completers__.items():
             new[k] = v
         new[name] = func
@@ -21,11 +25,11 @@ def _add_one_completer(name, func, loc='end'):
         direction, rel = loc[0], loc[1:]
         found = False
         for (k, v) in builtins.__xonsh_completers__.items():
-            if rel == k and direction == '<':
+            if rel == k and direction == "<":
                 new[name] = func
                 found = True
             new[k] = v
-            if rel == k and direction == '>':
+            if rel == k and direction == ">":
                 new[name] = func
                 found = True
         if not found:
@@ -34,108 +38,119 @@ def _add_one_completer(name, func, loc='end'):
     builtins.__xonsh_completers__.update(new)
 
 
-def _list_completers(args, stdin=None):
+def _list_completers(args, stdin=None, stack=None):
     o = "Registered Completer Functions: \n"
     _comp = builtins.__xonsh_completers__
     ml = max((len(i) for i in _comp), default=0)
     _strs = []
     for c in _comp:
         if _comp[c].__doc__ is None:
-            doc = 'No description provided'
+            doc = "No description provided"
         else:
-            doc = ' '.join(_comp[c].__doc__.split())
+            doc = " ".join(_comp[c].__doc__.split())
         doc = justify(doc, 80, ml + 3)
-        _strs.append('{: >{}} : {}'.format(c, ml, doc))
-    return o + '\n'.join(_strs) + '\n'
+        _strs.append("{: >{}} : {}".format(c, ml, doc))
+    return o + "\n".join(_strs) + "\n"
 
 
-def _remove_completer(args, stdin=None):
+def _remove_completer(args, stdin=None, stack=None):
     err = None
     if len(args) != 1:
         err = "completer remove takes exactly 1 argument."
     else:
         name = args[0]
         if name not in builtins.__xonsh_completers__:
-            err = ("The name %s is not a registered "
-                   "completer function.") % name
+            err = ("The name %s is not a registered " "completer function.") % name
     if err is None:
         del builtins.__xonsh_completers__[name]
         return
     else:
-        return None, err + '\n', 1
+        return None, err + "\n", 1
 
 
-def _register_completer(args, stdin=None):
+def _register_completer(args, stdin=None, stack=None):
     err = None
     if len(args) not in {2, 3}:
-        err = ("completer add takes either 2 or 3 arguments.\n"
-               "For help, run:  completer help add")
+        err = (
+            "completer add takes either 2 or 3 arguments.\n"
+            "For help, run:  completer help add"
+        )
     else:
         name = args[0]
-        func = args[1]
+        func_name = args[1]
         if name in builtins.__xonsh_completers__:
-            err = ("The name %s is already a registered "
-                   "completer function.") % name
+            err = ("The name %s is already a registered " "completer function.") % name
         else:
-            if func in builtins.__xonsh_ctx__:
-                if not callable(builtins.__xonsh_ctx__[func]):
-                    err = "%s is not callable" % func
+            if func_name in builtins.__xonsh_ctx__:
+                func = builtins.__xonsh_ctx__[func_name]
+                if not callable(func):
+                    err = "%s is not callable" % func_name
             else:
-                err = "No such function: %s" % func
+                for frame_info in stack:
+                    frame = frame_info[0]
+                    if func_name in frame.f_locals:
+                        func = frame.f_locals[func_name]
+                        break
+                    elif func_name in frame.f_globals:
+                        func = frame.f_globals[func_name]
+                        break
+                else:
+                    err = "No such function: %s" % func_name
     if err is None:
         position = "start" if len(args) == 2 else args[2]
-        func = builtins.__xonsh_ctx__[func]
         _add_one_completer(name, func, position)
     else:
-        return None, err + '\n', 1
+        return None, err + "\n", 1
 
 
-def completer_alias(args, stdin=None):
+def completer_alias(args, stdin=None, stdout=None, stderr=None, spec=None, stack=None):
     err = None
-    if len(args) == 0 or args[0] not in (VALID_ACTIONS | {'help'}):
-        err = ('Please specify an action.  Valid actions are: '
-               '"add", "remove", "list", or "help".')
-    elif args[0] == 'help':
+    if len(args) == 0 or args[0] not in (VALID_ACTIONS | {"help"}):
+        err = (
+            "Please specify an action.  Valid actions are: "
+            '"add", "remove", "list", or "help".'
+        )
+    elif args[0] == "help":
         if len(args) == 1 or args[1] not in VALID_ACTIONS:
-            return ('Valid actions are: add, remove, list.  For help with a '
-                    'specific action, run: completer help ACTION\n')
-        elif args[1] == 'add':
+            return (
+                "Valid actions are: add, remove, list.  For help with a "
+                "specific action, run: completer help ACTION\n"
+            )
+        elif args[1] == "add":
             return COMPLETER_ADD_HELP_STR
-        elif args[1] == 'remove':
+        elif args[1] == "remove":
             return COMPLETER_REMOVE_HELP_STR
-        elif args[1] == 'list':
+        elif args[1] == "list":
             return COMPLETER_LIST_HELP_STR
 
     if err is not None:
-        return None, err + '\n', 1
+        return None, err + "\n", 1
 
-    if args[0] == 'add':
+    if args[0] == "add":
         func = _register_completer
-    elif args[0] == 'remove':
+    elif args[0] == "remove":
         func = _remove_completer
-    elif args[0] == 'list':
+    elif args[0] == "list":
         func = _list_completers
-    return func(args[1:], stdin=stdin)
+    return func(args[1:], stdin=stdin, stack=stack)
 
-COMPLETER_LIST_HELP_STR = """
-completer list: list the active completers, in order
+
+COMPLETER_LIST_HELP_STR = """completer list: ordered list the active completers
 
 Usage:
     completer remove
-""".lstrip()
+"""
 
-COMPLETER_REMOVE_HELP_STR = """
-completer remove: removes a completer from xonsh
+COMPLETER_REMOVE_HELP_STR = """completer remove: removes a completer from xonsh
 
 Usage:
     completer remove NAME
 
 NAME is a unique name of a completer (run "completer list" to see the current
      completers in order)
-""".lstrip()
+"""
 
-COMPLETER_ADD_HELP_STR = """
-completer add: adds a new completer to xonsh
+COMPLETER_ADD_HELP_STR = """completer add: adds a new completer to xonsh
 
 Usage:
     completer add NAME FUNC [POS]
@@ -172,4 +187,4 @@ POS (optional) is a position into the list of completers at which the new
                  be added before the completer named KEY
 
      If POS is not provided, the default value is "start"
-""".lstrip()
+"""
