@@ -15,11 +15,24 @@ from xonsh.completers.tools import get_filter_function
 def PATTERN_NEED_QUOTES():
     pattern = r'\s`\$\{\}\,\*\(\)"\'\?&#'
     if xp.ON_WINDOWS:
-        pattern += '%'
-    pattern = '[' + pattern + ']' + r'|\band\b|\bor\b'
+        pattern += "%"
+    pattern = "[" + pattern + "]" + r"|\band\b|\bor\b"
     return re.compile(pattern)
 
 
+def cd_in_command(line):
+    """Returns True if "cd" is a token in the line, False otherwise."""
+    lexer = builtins.__xonsh_execer__.parser.lexer
+    lexer.reset()
+    lexer.input(line)
+    have_cd = False
+    for tok in lexer:
+        if tok.type == "NAME" and tok.value == "cd":
+            have_cd = True
+            break
+    return have_cd
+
+
 def _path_from_partial_string(inp, pos=None):
     if pos is None:
         pos = len(inp)
@@ -33,23 +46,24 @@ def _path_from_partial_string(inp, pos=None):
     else:
         if endix != pos:
             _test = partial[endix:pos]
-            if not any(i == ' ' for i in _test):
+            if not any(i == " " for i in _test):
                 _post = _test
             else:
                 return None
         string = partial[startix:endix]
-    end = xt.RE_STRING_START.sub('', quote)
+    end = xt.RE_STRING_START.sub("", quote)
     _string = string
     if not _string.endswith(end):
         _string = _string + end
     try:
         val = ast.literal_eval(_string)
-    except SyntaxError:
+    except (SyntaxError, ValueError):
         return None
     if isinstance(val, bytes):
         env = builtins.__xonsh_env__
-        val = val.decode(encoding=env.get('XONSH_ENCODING'),
-                         errors=env.get('XONSH_ENCODING_ERRORS'))
+        val = val.decode(
+            encoding=env.get("XONSH_ENCODING"), errors=env.get("XONSH_ENCODING_ERRORS")
+        )
     return string + _post, val + _post, quote, end
 
 
@@ -59,16 +73,16 @@ def _normpath(p):
     and '/' at the end. On windows it does the same with backslashes
     """
     initial_dotslash = p.startswith(os.curdir + os.sep)
-    initial_dotslash |= (xp.ON_WINDOWS and p.startswith(os.curdir + os.altsep))
+    initial_dotslash |= xp.ON_WINDOWS and p.startswith(os.curdir + os.altsep)
     p = p.rstrip()
     trailing_slash = p.endswith(os.sep)
-    trailing_slash |= (xp.ON_WINDOWS and p.endswith(os.altsep))
+    trailing_slash |= xp.ON_WINDOWS and p.endswith(os.altsep)
     p = os.path.normpath(p)
-    if initial_dotslash and p != '.':
+    if initial_dotslash and p != ".":
         p = os.path.join(os.curdir, p)
     if trailing_slash:
-        p = os.path.join(p, '')
-    if xp.ON_WINDOWS and builtins.__xonsh_env__.get('FORCE_POSIX_PATHS'):
+        p = os.path.join(p, "")
+    if xp.ON_WINDOWS and builtins.__xonsh_env__.get("FORCE_POSIX_PATHS"):
         p = p.replace(os.sep, os.altsep)
     return p
 
@@ -84,22 +98,22 @@ def _startswithnorm(x, start, startlow=None):
 
 
 def _env(prefix):
-    if prefix.startswith('$'):
+    if prefix.startswith("$"):
         key = prefix[1:]
-        return {'$' + k
-                for k in builtins.__xonsh_env__
-                if get_filter_function()(k, key)}
+        return {
+            "$" + k for k in builtins.__xonsh_env__ if get_filter_function()(k, key)
+        }
     return ()
 
 
 def _dots(prefix):
     slash = xt.get_sep()
-    if slash == '\\':
-        slash = ''
-    if prefix in {'', '.'}:
-        return ('.'+slash, '..'+slash)
-    elif prefix == '..':
-        return ('..'+slash,)
+    if slash == "\\":
+        slash = ""
+    if prefix in {"", "."}:
+        return ("." + slash, ".." + slash)
+    elif prefix == "..":
+        return (".." + slash,)
     else:
         return ()
 
@@ -107,12 +121,13 @@ def _dots(prefix):
 def _add_cdpaths(paths, prefix):
     """Completes current prefix using CDPATH"""
     env = builtins.__xonsh_env__
-    csc = env.get('CASE_SENSITIVE_COMPLETIONS')
-    glob_sorted = env.get('GLOB_SORTED')
-    for cdp in env.get('CDPATH'):
-        test_glob = os.path.join(cdp, prefix) + '*'
-        for s in xt.iglobpath(test_glob, ignore_case=(not csc),
-                              sort_result=glob_sorted):
+    csc = env.get("CASE_SENSITIVE_COMPLETIONS")
+    glob_sorted = env.get("GLOB_SORTED")
+    for cdp in env.get("CDPATH"):
+        test_glob = os.path.join(cdp, prefix) + "*"
+        for s in xt.iglobpath(
+            test_glob, ignore_case=(not csc), sort_result=glob_sorted
+        ):
             if os.path.isdir(s):
                 paths.add(os.path.basename(s))
 
@@ -129,39 +144,39 @@ def _quote_to_use(x):
 def _quote_paths(paths, start, end, append_end=True):
     expand_path = builtins.__xonsh_expand_path__
     out = set()
-    space = ' '
-    backslash = '\\'
-    double_backslash = '\\\\'
+    space = " "
+    backslash = "\\"
+    double_backslash = "\\\\"
     slash = xt.get_sep()
     orig_start = start
     orig_end = end
     # quote on all or none, to make readline completes to max prefix
     need_quotes = any(
-        re.search(PATTERN_NEED_QUOTES, x) or
-        (backslash in x and slash != backslash)
-        for x in paths)
+        re.search(PATTERN_NEED_QUOTES, x) or (backslash in x and slash != backslash)
+        for x in paths
+    )
 
     for s in paths:
         start = orig_start
         end = orig_end
-        if start == '' and need_quotes:
+        if start == "" and need_quotes:
             start = end = _quote_to_use(s)
         if os.path.isdir(expand_path(s)):
             _tail = slash
-        elif end == '':
+        elif end == "":
             _tail = space
         else:
-            _tail = ''
-        if start != '' and 'r' not in start and backslash in s:
-            start = 'r%s' % start
+            _tail = ""
+        if start != "" and "r" not in start and backslash in s:
+            start = "r%s" % start
         s = s + _tail
-        if end != '':
+        if end != "":
             if "r" not in start.lower():
                 s = s.replace(backslash, double_backslash)
             if s.endswith(backslash) and not s.endswith(double_backslash):
                 s += backslash
         if end in s:
-            s = s.replace(end, ''.join('\\%s' % i for i in end))
+            s = s.replace(end, "".join("\\%s" % i for i in end))
         s = start + s + end if append_end else start + s
         out.add(s)
     return out, need_quotes
@@ -170,12 +185,12 @@ def _quote_paths(paths, start, end, append_end=True):
 def _joinpath(path):
     # convert our tuple representation back into a string representing a path
     if path is None:
-        return ''
+        return ""
     elif len(path) == 0:
-        return ''
-    elif path == ('',):
+        return ""
+    elif path == ("",):
         return xt.get_sep()
-    elif path[0] == '':
+    elif path[0] == "":
         return xt.get_sep() + _normpath(os.path.join(*path))
     else:
         return _normpath(os.path.join(*path))
@@ -186,7 +201,7 @@ def _splitpath(path):
     # if this tuple starts with '', it means that the path was an absolute path
     path = _normpath(path)
     if path.startswith(xt.get_sep()):
-        pre = ('', )
+        pre = ("",)
     else:
         pre = ()
     return pre + _splitpath_helper(path, ())
@@ -195,7 +210,7 @@ def _splitpath(path):
 def _splitpath_helper(path, sofar=()):
     folder, path = os.path.split(path)
     if path:
-        sofar = sofar + (path, )
+        sofar = sofar + (path,)
     if not folder or folder == xt.get_sep():
         return sofar[::-1]
     elif xp.ON_WINDOWS and not path:
@@ -235,21 +250,21 @@ def _subsequence_match_iter(ref, typed):
 
 def _expand_one(sofar, nextone, csc):
     out = set()
-    glob_sorted = builtins.__xonsh_env__.get('GLOB_SORTED')
+    glob_sorted = builtins.__xonsh_env__.get("GLOB_SORTED")
     for i in sofar:
-        _glob = os.path.join(_joinpath(i), '*') if i is not None else '*'
+        _glob = os.path.join(_joinpath(i), "*") if i is not None else "*"
         for j in xt.iglobpath(_glob, sort_result=glob_sorted):
             j = os.path.basename(j)
             if subsequence_match(j, nextone, csc):
-                out.add((i or ()) + (j, ))
+                out.add((i or ()) + (j,))
     return out
 
 
 def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
     """Completes based on a path name."""
     # string stuff for automatic quoting
-    path_str_start = ''
-    path_str_end = ''
+    path_str_start = ""
+    path_str_end = ""
     append_end = True
     p = _path_from_partial_string(line, end)
     lprefix = len(prefix)
@@ -260,24 +275,23 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
         path_str_end = p[3]
         if len(line) >= end + 1 and line[end] == path_str_end:
             append_end = False
-    tilde = '~'
+    tilde = "~"
     paths = set()
     env = builtins.__xonsh_env__
-    csc = env.get('CASE_SENSITIVE_COMPLETIONS')
-    glob_sorted = env.get('GLOB_SORTED')
+    csc = env.get("CASE_SENSITIVE_COMPLETIONS")
+    glob_sorted = env.get("GLOB_SORTED")
     prefix = glob.escape(prefix)
-    for s in xt.iglobpath(prefix + '*', ignore_case=(not csc),
-                          sort_result=glob_sorted):
+    for s in xt.iglobpath(prefix + "*", ignore_case=(not csc), sort_result=glob_sorted):
         paths.add(s)
-    if len(paths) == 0 and env.get('SUBSEQUENCE_PATH_COMPLETION'):
+    if len(paths) == 0 and env.get("SUBSEQUENCE_PATH_COMPLETION"):
         # this block implements 'subsequence' matching, similar to fish and zsh.
         # matches are based on subsequences, not substrings.
         # e.g., ~/u/ro completes to ~/lou/carcolh
         # see above functions for details.
         p = _splitpath(os.path.expanduser(prefix))
         if len(p) != 0:
-            if p[0] == '':
-                basedir = ('', )
+            if p[0] == "":
+                basedir = ("",)
                 p = p[1:]
             else:
                 basedir = None
@@ -285,28 +299,28 @@ def complete_path(prefix, line, start, end, ctx, cdpath=True, filtfunc=None):
             for i in p:
                 matches_so_far = _expand_one(matches_so_far, i, csc)
             paths |= {_joinpath(i) for i in matches_so_far}
-    if len(paths) == 0 and env.get('FUZZY_PATH_COMPLETION'):
-        threshold = env.get('SUGGEST_THRESHOLD')
-        for s in xt.iglobpath(os.path.dirname(prefix) + '*',
-                              ignore_case=(not csc),
-                              sort_result=glob_sorted):
+    if len(paths) == 0 and env.get("FUZZY_PATH_COMPLETION"):
+        threshold = env.get("SUGGEST_THRESHOLD")
+        for s in xt.iglobpath(
+            os.path.dirname(prefix) + "*",
+            ignore_case=(not csc),
+            sort_result=glob_sorted,
+        ):
             if xt.levenshtein(prefix, s, threshold) < threshold:
                 paths.add(s)
     if tilde in prefix:
         home = os.path.expanduser(tilde)
         paths = {s.replace(home, tilde) for s in paths}
-    if cdpath:
+    if cdpath and cd_in_command(line):
         _add_cdpaths(paths, prefix)
     paths = set(filter(filtfunc, paths))
-    paths, _ = _quote_paths({_normpath(s) for s in paths},
-                            path_str_start,
-                            path_str_end,
-                            append_end)
+    paths, _ = _quote_paths(
+        {_normpath(s) for s in paths}, path_str_start, path_str_end, append_end
+    )
     paths.update(filter(filtfunc, _dots(prefix)))
     paths.update(filter(filtfunc, _env(prefix)))
     return paths, lprefix
 
 
 def complete_dir(prefix, line, start, end, ctx, cdpath=False):
-    return complete_path(prefix, line, start, end, cdpath,
-                         filtfunc=os.path.isdir)
+    return complete_path(prefix, line, start, end, cdpath, filtfunc=os.path.isdir)
