@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-"""Prompt formatter for simple version control branchs"""
+"""Prompt formatter for simple version control branches"""
 # pylint:disable=no-member, invalid-name
 
 import os
@@ -13,18 +13,20 @@ import xonsh.tools as xt
 
 
 def _get_git_branch(q):
+    denv = builtins.__xonsh_env__.detype()
     try:
-        branches = xt.decode_bytes(subprocess.check_output(
-            ['git', 'branch'],
-            stderr=subprocess.DEVNULL
-        )).splitlines()
+        branches = xt.decode_bytes(
+            subprocess.check_output(
+                ["git", "branch"], env=denv, stderr=subprocess.DEVNULL
+            )
+        ).splitlines()
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
     else:
         for branch in branches:
-            if not branch.startswith('* '):
+            if not branch.startswith("* "):
                 continue
-            elif branch.endswith(')'):
+            elif branch.endswith(")"):
                 branch = branch.split()[-1][:-1]
             else:
                 branch = branch.split()[-1]
@@ -40,7 +42,7 @@ def get_git_branch():
     be determined (timeout, not in a git repo, etc.) then this returns None.
     """
     branch = None
-    timeout = builtins.__xonsh_env__.get('VC_BRANCH_TIMEOUT')
+    timeout = builtins.__xonsh_env__.get("VC_BRANCH_TIMEOUT")
     q = queue.Queue()
 
     t = threading.Thread(target=_get_git_branch, args=(q,))
@@ -54,11 +56,11 @@ def get_git_branch():
 
 
 def _get_hg_root(q):
-    _curpwd = builtins.__xonsh_env__['PWD']
+    _curpwd = builtins.__xonsh_env__["PWD"]
     while True:
         if not os.path.isdir(_curpwd):
             return False
-        if any([b.name == '.hg' for b in os.scandir(_curpwd)]):
+        if any([b.name == ".hg" for b in xt.scandir(_curpwd)]):
             q.put(_curpwd)
             break
         else:
@@ -73,7 +75,7 @@ def get_hg_branch(root=None):
     return None if not in a repo or subprocess.TimeoutExpired if timed out.
     """
     env = builtins.__xonsh_env__
-    timeout = env['VC_BRANCH_TIMEOUT']
+    timeout = env["VC_BRANCH_TIMEOUT"]
     q = queue.Queue()
     t = threading.Thread(target=_get_hg_root, args=(q,))
     t.start()
@@ -82,24 +84,25 @@ def get_hg_branch(root=None):
         root = q.get_nowait()
     except queue.Empty:
         return None
-    if env.get('VC_HG_SHOW_BRANCH'):
+    if env.get("VC_HG_SHOW_BRANCH"):
         # get branch name
-        branch_path = os.path.sep.join([root, '.hg', 'branch'])
+        branch_path = os.path.sep.join([root, ".hg", "branch"])
         if os.path.exists(branch_path):
-            with open(branch_path, 'r') as branch_file:
+            with open(branch_path, "r") as branch_file:
                 branch = branch_file.read()
         else:
-            branch = 'default'
+            branch = "default"
     else:
-        branch = ''
+        branch = ""
     # add bookmark, if we can
-    bookmark_path = os.path.sep.join([root, '.hg', 'bookmarks.current'])
+    bookmark_path = os.path.sep.join([root, ".hg", "bookmarks.current"])
     if os.path.exists(bookmark_path):
-        with open(bookmark_path, 'r') as bookmark_file:
+        with open(bookmark_path, "r") as bookmark_file:
             active_bookmark = bookmark_file.read()
-        if env.get('VC_HG_SHOW_BRANCH') is True:
-            branch = "{0}, {1}".format(*(b.strip(os.linesep) for b in
-                                         (branch, active_bookmark)))
+        if env.get("VC_HG_SHOW_BRANCH") is True:
+            branch = "{0}, {1}".format(
+                *(b.strip(os.linesep) for b in (branch, active_bookmark))
+            )
         else:
             branch = active_bookmark.strip(os.linesep)
     else:
@@ -112,18 +115,20 @@ _FIRST_BRANCH_TIMEOUT = True
 
 def _first_branch_timeout_message():
     global _FIRST_BRANCH_TIMEOUT
-    sbtm = builtins.__xonsh_env__['SUPPRESS_BRANCH_TIMEOUT_MESSAGE']
+    sbtm = builtins.__xonsh_env__["SUPPRESS_BRANCH_TIMEOUT_MESSAGE"]
     if not _FIRST_BRANCH_TIMEOUT or sbtm:
         return
     _FIRST_BRANCH_TIMEOUT = False
-    print('xonsh: branch timeout: computing the branch name, color, or both '
-          'timed out while formatting the prompt. You may avoid this by '
-          'increasing the value of $VC_BRANCH_TIMEOUT or by removing branch '
-          'fields, like {curr_branch}, from your $PROMPT. See the FAQ '
-          'for more details. This message will be suppressed for the remainder '
-          'of this session. To suppress this message permanently, set '
-          '$SUPPRESS_BRANCH_TIMEOUT_MESSAGE = True in your xonshrc file.',
-          file=sys.stderr)
+    print(
+        "xonsh: branch timeout: computing the branch name, color, or both "
+        "timed out while formatting the prompt. You may avoid this by "
+        "increasing the value of $VC_BRANCH_TIMEOUT or by removing branch "
+        "fields, like {curr_branch}, from your $PROMPT. See the FAQ "
+        "for more details. This message will be suppressed for the remainder "
+        "of this session. To suppress this message permanently, set "
+        "$SUPPRESS_BRANCH_TIMEOUT_MESSAGE = True in your xonshrc file.",
+        file=sys.stderr,
+    )
 
 
 def current_branch():
@@ -136,30 +141,31 @@ def current_branch():
     cmds = builtins.__xonsh_commands_cache__
     # check for binary only once
     if cmds.is_empty():
-        has_git = bool(cmds.locate_binary('git', ignore_alias=True))
-        has_hg = bool(cmds.locate_binary('hg', ignore_alias=True))
+        has_git = bool(cmds.locate_binary("git", ignore_alias=True))
+        has_hg = bool(cmds.locate_binary("hg", ignore_alias=True))
     else:
-        has_git = bool(cmds.lazy_locate_binary('git', ignore_alias=True))
-        has_hg = bool(cmds.lazy_locate_binary('hg', ignore_alias=True))
+        has_git = bool(cmds.lazy_locate_binary("git", ignore_alias=True))
+        has_hg = bool(cmds.lazy_locate_binary("hg", ignore_alias=True))
     if has_git:
         branch = get_git_branch()
     if not branch and has_hg:
         branch = get_hg_branch()
     if isinstance(branch, subprocess.TimeoutExpired):
-        branch = '<branch-timeout>'
+        branch = "<branch-timeout>"
         _first_branch_timeout_message()
     return branch or None
 
 
 def _git_dirty_working_directory(q, include_untracked):
     status = None
+    denv = builtins.__xonsh_env__.detype()
     try:
-        cmd = ['git', 'status', '--porcelain']
+        cmd = ["git", "status", "--porcelain"]
         if include_untracked:
-            cmd.append('--untracked-files=normal')
+            cmd.append("--untracked-files=normal")
         else:
-            cmd.append('--untracked-files=no')
-        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL)
+            cmd.append("--untracked-files=no")
+        status = subprocess.check_output(cmd, stderr=subprocess.DEVNULL, env=denv)
     except (subprocess.CalledProcessError, OSError, FileNotFoundError):
         q.put(None)
     if status is not None:
@@ -172,8 +178,9 @@ def git_dirty_working_directory(include_untracked=False):
     """
     timeout = builtins.__xonsh_env__.get("VC_BRANCH_TIMEOUT")
     q = queue.Queue()
-    t = threading.Thread(target=_git_dirty_working_directory,
-                         args=(q, include_untracked))
+    t = threading.Thread(
+        target=_git_dirty_working_directory, args=(q, include_untracked)
+    )
     t.start()
     t.join(timeout=timeout)
     try:
@@ -187,19 +194,26 @@ def hg_dirty_working_directory():
     If this cannot be determined, None is returned.
     """
     env = builtins.__xonsh_env__
-    cwd = env['PWD']
+    cwd = env["PWD"]
     denv = env.detype()
-    vcbt = env['VC_BRANCH_TIMEOUT']
+    vcbt = env["VC_BRANCH_TIMEOUT"]
     # Override user configurations settings and aliases
-    denv['HGRCPATH'] = ''
+    denv["HGRCPATH"] = ""
     try:
-        s = subprocess.check_output(['hg', 'identify', '--id'],
-                                    stderr=subprocess.PIPE, cwd=cwd,
-                                    timeout=vcbt, universal_newlines=True,
-                                    env=denv)
-        return s.strip(os.linesep).endswith('+')
-    except (subprocess.CalledProcessError, subprocess.TimeoutExpired,
-            FileNotFoundError):
+        s = subprocess.check_output(
+            ["hg", "identify", "--id"],
+            stderr=subprocess.PIPE,
+            cwd=cwd,
+            timeout=vcbt,
+            universal_newlines=True,
+            env=denv,
+        )
+        return s.strip(os.linesep).endswith("+")
+    except (
+        subprocess.CalledProcessError,
+        subprocess.TimeoutExpired,
+        FileNotFoundError,
+    ):
         return None
 
 
@@ -210,9 +224,9 @@ def dirty_working_directory():
     """
     dwd = None
     cmds = builtins.__xonsh_commands_cache__
-    if cmds.lazy_locate_binary('git'):
+    if cmds.lazy_locate_binary("git", ignore_alias=True):
         dwd = git_dirty_working_directory()
-    if cmds.lazy_locate_binary('hg') and dwd is None:
+    if cmds.lazy_locate_binary("hg", ignore_alias=True) and dwd is None:
         dwd = hg_dirty_working_directory()
     return dwd
 
@@ -224,23 +238,23 @@ def branch_color():
     """
     dwd = dirty_working_directory()
     if dwd is None:
-        color = '{BOLD_INTENSE_YELLOW}'
+        color = "{BOLD_INTENSE_YELLOW}"
     elif dwd:
-        color = '{BOLD_INTENSE_RED}'
+        color = "{BOLD_INTENSE_RED}"
     else:
-        color = '{BOLD_INTENSE_GREEN}'
+        color = "{BOLD_INTENSE_GREEN}"
     return color
 
 
 def branch_bg_color():
     """Return red if the current branch is dirty, yellow if the dirtiness can
-    not be determined, and green if it clean. These are bacground colors.
+    not be determined, and green if it clean. These are background colors.
     """
     dwd = dirty_working_directory()
     if dwd is None:
-        color = '{BACKGROUND_YELLOW}'
+        color = "{BACKGROUND_YELLOW}"
     elif dwd:
-        color = '{BACKGROUND_RED}'
+        color = "{BACKGROUND_RED}"
     else:
-        color = '{BACKGROUND_GREEN}'
+        color = "{BACKGROUND_GREEN}"
     return color
