@@ -1,15 +1,13 @@
 # -*- coding: utf-8 -*-
 """History object for use with prompt_toolkit."""
-import time
 import builtins
 from threading import Thread
 
 import prompt_toolkit.history
-from xonsh import lazyjson
 
 
 class PromptToolkitHistory(prompt_toolkit.history.History):
-    """History class that implements the promt-toolkit history interface
+    """History class that implements the prompt-toolkit history interface
     with the xonsh backend.
     """
 
@@ -35,7 +33,6 @@ class PromptToolkitHistory(prompt_toolkit.history.History):
 
 
 class PromptToolkitHistoryAdder(Thread):
-
     def __init__(self, ptkhist, wait_for_gc=True, *args, **kwargs):
         """Thread responsible for adding inputs from history to the current
         prompt-toolkit history instance. May wait for the history garbage
@@ -49,34 +46,24 @@ class PromptToolkitHistoryAdder(Thread):
 
     def run(self):
         hist = builtins.__xonsh_history__
+        if hist is None:
+            return
         buf = None
         ptkhist = self.ptkhist
-        while self.wait_for_gc and hist.gc.is_alive():
-            time.sleep(0.011)  # gc sleeps for 0.01 secs, sleep a beat longer
-        files = hist.gc.files()
-        for _, _, f in files:
-            try:
-                lj = lazyjson.LazyJSON(f, reopen=False)
-                for cmd in lj['cmds']:
-                    line = cmd['inp'].rstrip()
-                    if line == 'EOF':
+        for cmd in hist.all_items():
+            line = cmd["inp"].rstrip()
+            if len(ptkhist) == 0 or line != ptkhist[-1]:
+                ptkhist.append(line)
+                if buf is None:
+                    buf = self._buf()
+                    if buf is None:
                         continue
-                    if len(ptkhist) == 0 or line != ptkhist[-1]:
-                        ptkhist.append(line)
-                        if buf is None:
-                            buf = self._buf()
-                            if buf is None:
-                                continue
-                        buf.reset(initial_document=buf.document)
-                lj.close()
-            except (IOError, OSError):
-                continue
+                buf.reset(initial_document=buf.document)
 
     def _buf(self):
         # Thread-safe version of
         # buf = builtins.__xonsh_shell__.shell.prompter.cli.application.buffer
-        path = ['__xonsh_shell__', 'shell', 'prompter', 'cli', 'application',
-                'buffer']
+        path = ["__xonsh_shell__", "shell", "prompter", "cli", "application", "buffer"]
         buf = builtins
         for a in path:
             buf = getattr(buf, a, None)
